<!doctype html>
<title>Upload new File</title>
<h1>Upload new File</h1>
<form action="" method=post enctype=multipart/form-data>
  <p>
    <input type=file name=file><br/>
    Secret Key <input type=text name=key><br/>
    <input type=submit value=Upload>
  </p>
</form>
