.editor {
    margin: 0;
    position: absolute;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
}

.box_header{
  background: #E4E4E4;
  width: 100%;
  height: 30px;
  color: #737373;
  border-bottom: 4px solid #7FAED3;
  min-width: 936px;
}
.box_header ul{float: left; padding-top: 2px; text-shadow: 0px 1px #F1F1F1;}
.box_header li{float: left;border: 1px solid #AAB8C2;padding: 1px 5px 1px 0;margin-left: 5px; text-indent: 5px;}
/*.box_header li:last-child{border:none}*/
.box_header li > span{cursor: pointer; height: 20px; display: block;line-height: 20px;font-weight: bold;padding: 1px;}
.box_header li:hover{border: 1px solid #57A1D6;background-color: #C7C7C7;}
.box_header li:last-child{margin-right: 5px;}
.box_header li>a{font-weight: bold; font-size:1em;display: block;padding: 2px;}
.save_btn{background: url(../images/icon_save.png) center right no-repeat;width: 25px;}
.expand_editor{background: url(../images/fullscreen.png) center right no-repeat;width: 23px;}
.e_expanded{background: url(../images/fullscreen_exit.png) center right no-repeat;}
.swith_btn{background: url(../images/gnome-session-switch.png) center right no-repeat;width: 105px;}
.flist_btn{background: url(../images/list2_down.png) center right no-repeat;width: 26px;}
.fmenu_btn{background: url(../images/ui_menu_blue.png) center right no-repeat;width: 58px;}
.shell_btn{background: url(../images/terminal.png) center right no-repeat;width: 62px;}

#tabControl{overflow: hidden;}
#tabControl .item{float:left; min-width: 60px; background: #D5D5D5; height: 22px; padding-top: 8px; font-size:1em;
border-left:1px #E4E4E4 solid; cursor:pointer; color: #5C7077; text-shadow: 0px 1px #E6E6E6; position: relative;}
#tabControl .item:hover{background: #C7C7C7;}
#tabControl.item:last-child{margin-right:none; overflow: hidden}
#tabControl .active{background: #7FAED3; color: #fff; text-shadow: none;}
#tabControl .active:hover{background: #7FAED3;}
#tabControl .item span{padding: 0 4px; display: block; float: left; text-overflow:clip; max-width: 126px; white-space:nowrap; overflow: hidden;}
#tabControl .active span.bt_close{color: #DBDBDB;}
#tabControl .active span.bt_close:hover{color: #fff;}
#tabContent pre {display: none;}
#tabContent pre.active {display: block;}
.item-hide{display:none;}

#shell-window {
  width: 974px;
  height: 350px;
  z-index: 10;
  border: 2px solid #6A93A0;
  border-top: none;
  background: #9bbed6;
  position: absolute;
  display: none;
  border-bottom-left-radius: 5px;
  border-bottom-right-radius: 5px;
}
#shell-result {
  width: 98%;
  height: 300px;
  margin-left: 1%;
  margin-right: 1%;
  margin-top: 5px;
  padding: 3px;
  background: #E4E4E4;
  color: #074A86;
  box-sizing: border-box;
  border: 2px inset #6A93A0;
  overflow: auto;
}
.runned-command
{ 
  font-weight: bold;
}
#shell-input {
  width: 96%;
  margin-left: 1%;
  margin-right: 1%;
  position: absolute;
  bottom: 10px;
  background: #E4E4E4;
  color: #074A86;
  border: inset 1px #678dad; 
}
